@@ -16,27 +16,24 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <net/if.h>
 
 #include "dnssd.h"
 #include "logging.h"
 #include "options.h"
+#include "capabilities.h"
+
+
 
 /*
- * 'dnssd_callback()' - Handle DNS-SD registration events.
+ * 'dnssd_callback()' - Handle DNS-SD registration events generic.
  */
 
 static void
 dnssd_callback(AvahiEntryGroup      *g,		/* I - Service */
-	       AvahiEntryGroupState state,	/* I - Registration state */
-	       void                 *context)	/* I - Printer */
+	       AvahiEntryGroupState state)	/* I - Registration state */
 {
-  (void)context;
-
-  if (g == NULL || (g_options.dnssd_data->ipp_ref != NULL &&
-		    g_options.dnssd_data->ipp_ref != g))
-    return;
-
   switch (state) {
   case AVAHI_ENTRY_GROUP_ESTABLISHED :
     /* The entry group has been established successfully */
@@ -57,6 +54,39 @@ dnssd_callback(AvahiEntryGroup      *g,		/* I - Service */
   }
 }
 
+/*
+ * 'dnssd_callback()' - Handle DNS-SD registration events ipp.
+ */
+
+static void
+dnssd_callback_ipp(AvahiEntryGroup      *g,	/* I - Service */
+	       AvahiEntryGroupState state,	/* I - Registration state */
+	       void                 *context)	/* I - Printer */
+{
+  (void)context;
+
+  if (g == NULL || (g_options.dnssd_data->ipp_ref != NULL &&
+		    g_options.dnssd_data->ipp_ref != g))
+    return;
+  dnssd_callback(g, state);
+}
+
+/*
+ * 'dnssd_callback()' - Handle DNS-SD registration events uscan.
+ */
+
+static void
+dnssd_callback_uscan(AvahiEntryGroup      *g,	/* I - Service */
+	       AvahiEntryGroupState state,	/* I - Registration state */
+	       void                 *context)	/* I - Printer */
+{
+  (void)context;
+
+  if (g == NULL || (g_options.dnssd_data->uscan_ref != NULL &&
+		    g_options.dnssd_data->uscan_ref != g))
+    return;
+  dnssd_callback(g, state);
+}
 
 /*
  * 'dnssd_client_cb()' - Client callback for Avahi.
@@ -94,6 +124,8 @@ dnssd_client_cb(AvahiClient      *c,		/* I - Client */
     NOTE("Dropping printer registration because of possible host name change.");
     if (g_options.dnssd_data->ipp_ref)
       avahi_entry_group_reset(g_options.dnssd_data->ipp_ref);
+    if (g_options.dnssd_data->uscan_ref)
+      avahi_entry_group_reset(g_options.dnssd_data->uscan_ref);
     break;
 
   case AVAHI_CLIENT_FAILURE:
@@ -121,10 +153,9 @@ dnssd_client_cb(AvahiClient      *c,		/* I - Client */
   }
 }
 
-int
-dnssd_init()
+int dnssd_init()
 {
-  int error;			/* Error code, if any */
+  int error; /* Error code, if any */
 
   g_options.dnssd_data = calloc(1, sizeof(dnssd_t));
   if (g_options.dnssd_data == NULL) {
@@ -134,16 +165,16 @@ dnssd_init()
   g_options.dnssd_data->DNSSDMaster = NULL;
   g_options.dnssd_data->DNSSDClient = NULL;
   g_options.dnssd_data->ipp_ref = NULL;
+  g_options.dnssd_data->uscan_ref = NULL;
 
   if ((g_options.dnssd_data->DNSSDMaster = avahi_threaded_poll_new()) == NULL) {
     ERR("Error: Unable to initialize DNS-SD.");
     goto fail;
   }
 
-  if ((g_options.dnssd_data->DNSSDClient =
-       avahi_client_new(avahi_threaded_poll_get(g_options.dnssd_data->DNSSDMaster),
-			AVAHI_CLIENT_NO_FAIL,
-			dnssd_client_cb, NULL, &error)) == NULL) {
+  if ((g_options.dnssd_data->DNSSDClient = avahi_client_new(
+           avahi_threaded_poll_get(g_options.dnssd_data->DNSSDMaster),
+           AVAHI_CLIENT_NO_FAIL, dnssd_client_cb, NULL, &error)) == NULL) {
     ERR("Error: Unable to initialize DNS-SD client.");
     goto fail;
   }
@@ -160,9 +191,8 @@ dnssd_init()
   return -1;
 }
 
-void
-dnssd_shutdown() {
-
+void dnssd_shutdown()
+{
   if (g_options.dnssd_data->DNSSDMaster) {
     avahi_threaded_poll_stop(g_options.dnssd_data->DNSSDMaster);
     dnssd_unregister();
@@ -182,12 +212,153 @@ dnssd_shutdown() {
   NOTE("DNS-SD shut down.");
 }
 
-/*
- * 'dnssd_register()' - Register a printer object via DNS-SD.
- */
+void * dnssd_escl_register(void *data)
+{
+  AvahiStringList *uscan_txt;             /* DNS-SD USCAN TXT record */
+  AvahiStringList *ipp_txt;             /* DNS-SD USCAN TXT record */
+  ippScanner      *scanner = NULL;
+  int             error;
+  char            temp[256];            /* Subtype service string */
+ 
+  ipp_txt = (AvahiStringList *)data;
 
-int
-dnssd_register(AvahiClient *c)
+  ippPrinter *printer = (ippPrinter *)calloc(1, sizeof(ippPrinter));
+  ipp_request(printer, g_options.real_port);
+  if (printer->adminurl)
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "adminurl=%s", printer->adminurl);
+  else
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "adminurl=%s", temp);
+  if (printer->uuid)
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "UUID=%s", printer->uuid);
+  if (printer->mopria_certified)
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "mopria-certified=%s", printer->mopria_certified);
+  if (printer->kind)
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "kind=%s", printer->kind);
+  if (printer->color)
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "Color=%s", printer->color);
+  if (printer->note)
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "note=%s", printer->note);
+  else
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "note=");
+  if (printer->ty) {
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "ty=%s", printer->ty);
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "product=(%s)", printer->ty);
+  }
+  if (printer->pdl)
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "pdl=%s", printer->pdl);
+  if (printer->urf)
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "URF=%s", printer->urf);
+  if (printer->papermax)
+     ipp_txt = avahi_string_list_add_printf(ipp_txt, "PaperMax=%s", printer->papermax);
+  if (printer->side)
+     ipp_txt = avahi_string_list_add_printf(ipp_txt, "Duplex=%s", printer->side);
+  if (printer->fax) {
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "Fax=%s", printer->fax);
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "rfo=ipp/faxout");
+  }
+  else
+    ipp_txt = avahi_string_list_add_printf(ipp_txt, "Fax=F");
+
+  NOTE("Printer TXT[\n\tadminurl=%s\n\tUUID=%s\t\n]\n", printer->adminurl, printer->uuid);
+
+ /*
+  * Then register the _ipp._tcp (IPP)...
+  */
+  error = avahi_entry_group_update_service_txt_strlst(
+      g_options.dnssd_data->ipp_ref,
+      (g_options.interface ? (int)if_nametoindex(g_options.interface)
+                           : AVAHI_IF_UNSPEC),
+      AVAHI_PROTO_UNSPEC, 0, g_options.dnssd_data->dnssd_name, "_ipp._tcp", NULL, ipp_txt);
+
+  if (error) {
+    ERR("Error registering %s as IPP printer (_ipp._tcp): %d", g_options.dnssd_data->dnssd_name,
+	error);
+  }
+  avahi_entry_group_commit(g_options.dnssd_data->ipp_ref);
+  avahi_string_list_free(ipp_txt);
+
+  snprintf(temp, sizeof(temp), "http://127.0.0.1:%d/", g_options.real_port);
+  scanner = (ippScanner*) calloc(1, sizeof(ippScanner));
+  if (is_scanner_present(scanner, g_options.real_port) == 0 || scanner == NULL)
+     goto noscanner;
+  /*
+   * Create the TXT record for scanner ...
+   */
+  uscan_txt = NULL;
+  if (scanner->representation)
+     uscan_txt = avahi_string_list_add_printf(uscan_txt, "representation=%s", scanner->representation);
+  else if (printer->representation)
+     uscan_txt = avahi_string_list_add_printf(uscan_txt, "representation=%s", printer->representation);
+  uscan_txt = avahi_string_list_add_printf(uscan_txt, "note=");
+  if (scanner->uuid)
+     uscan_txt = avahi_string_list_add_printf(uscan_txt, "UUID=%s", scanner->uuid);
+  else if (printer->uuid)
+     uscan_txt = avahi_string_list_add_printf(uscan_txt, "UUID=%s", printer->uuid);
+  if (scanner->adminurl)
+     uscan_txt = avahi_string_list_add_printf(uscan_txt, "adminurl=%s", scanner->adminurl);
+  else if (printer->adminurl)
+     uscan_txt = avahi_string_list_add_printf(uscan_txt, "adminurl=%s", printer->adminurl);
+  else
+     uscan_txt = avahi_string_list_add_printf(uscan_txt, "adminurl=%s", temp);
+  uscan_txt = avahi_string_list_add_printf(uscan_txt, "duplex=%s", scanner->duplex);
+  uscan_txt = avahi_string_list_add_printf(uscan_txt, "cs=%s", scanner->cs);
+  uscan_txt = avahi_string_list_add_printf(uscan_txt, "pdl=%s", scanner->pdl);
+  uscan_txt = avahi_string_list_add_printf(uscan_txt, "ty=%s", scanner->ty);
+  uscan_txt = avahi_string_list_add_printf(uscan_txt, "rs=eSCL");
+  uscan_txt = avahi_string_list_add_printf(uscan_txt, "vers=%s", scanner->vers);
+  uscan_txt = avahi_string_list_add_printf(uscan_txt, "txtvers=1");
+
+
+ /*
+  * Register _uscan._tcp (LPD) with port 0 to reserve the service name...
+  */
+
+  NOTE("Registering scanner %s on interface %s for DNS-SD broadcasting ...",
+       scanner->ty, g_options.interface);
+
+  if (g_options.dnssd_data->uscan_ref == NULL)
+    g_options.dnssd_data->uscan_ref =
+      avahi_entry_group_new(g_options.dnssd_data->DNSSDClient,
+			    dnssd_callback_uscan, NULL);
+
+  if (g_options.dnssd_data->uscan_ref == NULL) {
+    ERR("Could not establish Avahi entry group");
+    avahi_string_list_free(uscan_txt);
+    scanner = free_scanner(scanner);
+    goto noscanner;
+  }
+
+  error =
+    avahi_entry_group_add_service_strlst(g_options.dnssd_data->uscan_ref,
+					 (g_options.interface ?
+					  (int)if_nametoindex(g_options.interface) :
+					  AVAHI_IF_UNSPEC),
+					 AVAHI_PROTO_UNSPEC, 0,
+					 g_options.dnssd_data->dnssd_name,
+					 "_uscan._tcp", NULL, NULL,
+					 g_options.real_port, uscan_txt);
+  if (error) {
+    ERR("Error registering %s as Unix scanner (_uscan._tcp): %d", scanner->ty,
+	error);
+    scanner = free_scanner(scanner);
+    goto noscanner;
+  }else
+    NOTE("Registered %s as Unix scanner (_uscan._tcp).", scanner->ty);
+
+ /*
+  * Commit it scanner ...
+  */
+
+  avahi_entry_group_commit(g_options.dnssd_data->uscan_ref);
+
+  avahi_string_list_free(uscan_txt);
+  scanner = free_scanner(scanner);
+noscanner:
+  printer = free_printer(printer);
+  return 0;
+}
+
+int dnssd_register(AvahiClient *c)
 {
   AvahiStringList *ipp_txt;             /* DNS-SD IPP TXT record */
   char            temp[256];            /* Subtype service string */
@@ -203,18 +374,20 @@ dnssd_register(AvahiClient *c)
                   pclm = 0,
                   pdf = 0,
                   jpeg = 0;
-  char            has_duplex = 'U',
-                  has_color = 'U',
-                  has_copies = 'U';
   char            formats[1024];        /* I - Supported formats */
   char            *ptr;
   int             error;
+  pthread_t       thread_escl;
 
  /*
   * Parse the device ID for MFG, MDL, and CMD
   */
 
+  
   dev_id = strdup(g_options.device_id);
+  NOTE("%s", "=======================================");
+  NOTE("%s", dev_id);
+  NOTE("%s", "=======================================");
   if ((ptr = strcasestr(dev_id, "MFG:")) == NULL)
     if ((ptr = strcasestr(dev_id, "MANUFACTURER:")) == NULL) {
       ERR("No manufacturer info in device ID");
@@ -285,26 +458,6 @@ dnssd_register(AvahiClient *c)
 	   (pclm ? "application/PCLm," : ""),
 	   (jpeg ? "image/jpeg," : ""));
   formats[strlen(formats) - 1] = '\0';
-  if (urf) {
-    if ((ptr = strcasestr(urf, "DM")) != NULL &&
-	*(ptr + 2) >= '1' && *(ptr + 2) <= '4')
-      has_duplex = 'T';
-    else
-      has_duplex = 'F';
-    if ((ptr = strcasestr(urf, "CP")) != NULL) {
-      ptr += 2;
-      if ((*ptr >= '2' && *ptr <= '9') ||
-	  (*ptr == '1' && *(ptr + 1) >= '0' && *(ptr + 1) <= '9'))
-	has_copies = 'T';
-      else
-	has_copies = 'F';
-    } else
-      has_copies = 'F';
-    if ((ptr = strcasestr(urf, "RGB")) != NULL)
-      has_color = 'T';
-    else
-      has_color = 'F';
-  }
 
  /*
   * Additional printer properties
@@ -315,27 +468,17 @@ dnssd_register(AvahiClient *c)
     snprintf(dnssd_name, sizeof(dnssd_name), "%s [%s]", model, serial);
   else
     snprintf(dnssd_name, sizeof(dnssd_name), "%s", model);
-
+  g_options.dnssd_data->dnssd_name = strdup(dnssd_name);
  /*
-  * Create the TXT record...
+  * Create the TXT record for printer ...
   */
 
+//  UUID=cfe92100-67c4-11d4-a45f-f8d0273ebac3" "note=" "adminurl=http://EPSON3EBAC3.local.:80/PRESENTATION/BONJOUR"
+
   ipp_txt = NULL;
   ipp_txt = avahi_string_list_add_printf(ipp_txt, "rp=ipp/print");
-  ipp_txt = avahi_string_list_add_printf(ipp_txt, "ty=%s %s", make, model);
-  if (strcasecmp(g_options.interface, "lo") == 0)
-    ipp_txt = avahi_string_list_add_printf(ipp_txt, "adminurl=%s", temp);
-  ipp_txt = avahi_string_list_add_printf(ipp_txt, "product=(%s)", model);
-  ipp_txt = avahi_string_list_add_printf(ipp_txt, "pdl=%s", formats);
-  ipp_txt = avahi_string_list_add_printf(ipp_txt, "Color=%c", has_color);
-  ipp_txt = avahi_string_list_add_printf(ipp_txt, "Duplex=%c", has_duplex);
-  ipp_txt = avahi_string_list_add_printf(ipp_txt, "Copies=%c", has_copies);
   ipp_txt = avahi_string_list_add_printf(ipp_txt, "usb_MFG=%s", make);
   ipp_txt = avahi_string_list_add_printf(ipp_txt, "usb_MDL=%s", model);
-  if (urf)
-    ipp_txt = avahi_string_list_add_printf(ipp_txt, "URF=%s", urf);
-  else if (appleraster)
-    ipp_txt = avahi_string_list_add_printf(ipp_txt, "URF=CP1,IS1,MT1,RS300,SRGB24,W8");
   ipp_txt = avahi_string_list_add_printf(ipp_txt, "priority=60");
   ipp_txt = avahi_string_list_add_printf(ipp_txt, "txtvers=1");
   ipp_txt = avahi_string_list_add_printf(ipp_txt, "qtotal=1");
@@ -347,11 +490,12 @@ dnssd_register(AvahiClient *c)
 
   NOTE("Registering printer %s on interface %s for DNS-SD broadcasting ...",
        dnssd_name, g_options.interface);
-
+  if (c)
+     g_options.dnssd_data->DNSSDClient = c;
   if (g_options.dnssd_data->ipp_ref == NULL)
     g_options.dnssd_data->ipp_ref =
       avahi_entry_group_new((c ? c : g_options.dnssd_data->DNSSDClient),
-			    dnssd_callback, NULL);
+			    dnssd_callback_ipp, NULL);
 
   if (g_options.dnssd_data->ipp_ref == NULL) {
     ERR("Could not establish Avahi entry group");
@@ -359,15 +503,11 @@ dnssd_register(AvahiClient *c)
     return -1;
   }
 
-  error =
-    avahi_entry_group_add_service_strlst(g_options.dnssd_data->ipp_ref,
-					 (g_options.interface ?
-					  (int)if_nametoindex(g_options.interface) :
-					  AVAHI_IF_UNSPEC),
-					 AVAHI_PROTO_UNSPEC, 0,
-					 dnssd_name,
-					 "_printer._tcp", NULL, NULL, 0,
-					 NULL);
+  error = avahi_entry_group_add_service_strlst(
+      g_options.dnssd_data->ipp_ref,
+      (g_options.interface ? (int)if_nametoindex(g_options.interface)
+                           : AVAHI_IF_UNSPEC),
+      AVAHI_PROTO_UNSPEC, 0, dnssd_name, "_printer._tcp", NULL, NULL, 0, NULL);
   if (error)
     ERR("Error registering %s as Unix printer (_printer._tcp): %d", dnssd_name,
 	error);
@@ -378,94 +518,88 @@ dnssd_register(AvahiClient *c)
   * Then register the _ipp._tcp (IPP)...
   */
 
-  error =
-    avahi_entry_group_add_service_strlst(g_options.dnssd_data->ipp_ref,
-					 (g_options.interface ?
-					  (int)if_nametoindex(g_options.interface) :
-					  AVAHI_IF_UNSPEC),
-					 AVAHI_PROTO_UNSPEC, 0,
-					 dnssd_name,
-					 "_ipp._tcp", NULL, NULL, g_options.real_port,
-					 ipp_txt);
-  if (error)
+  error = avahi_entry_group_add_service_strlst(
+      g_options.dnssd_data->ipp_ref,
+      (g_options.interface ? (int)if_nametoindex(g_options.interface)
+                           : AVAHI_IF_UNSPEC),
+      AVAHI_PROTO_UNSPEC, 0, dnssd_name, "_ipp._tcp", NULL, NULL,
+      g_options.real_port, ipp_txt);
+
+  if (error) {
     ERR("Error registering %s as IPP printer (_ipp._tcp): %d", dnssd_name,
 	error);
-  else {
+  } else {
     NOTE("Registered %s as IPP printer (_ipp._tcp).", dnssd_name);
-    error =
-      avahi_entry_group_add_service_subtype(g_options.dnssd_data->ipp_ref,
-					    (g_options.interface ?
-					     (int)if_nametoindex(g_options.interface) :
-					     AVAHI_IF_UNSPEC),
-					    AVAHI_PROTO_UNSPEC, 0,
-					    dnssd_name,
-					    "_ipp._tcp", NULL,
-					    (appleraster && !pwgraster ?
-					     "_universal._sub._ipp._tcp" :
-					     "_print._sub._ipp._tcp"));
+    error = avahi_entry_group_add_service_subtype(
+        g_options.dnssd_data->ipp_ref,
+        (g_options.interface ? (int)if_nametoindex(g_options.interface)
+                             : AVAHI_IF_UNSPEC),
+        AVAHI_PROTO_UNSPEC, 0, dnssd_name, "_ipp._tcp", NULL,
+        (appleraster && !pwgraster ? "_universal._sub._ipp._tcp"
+                                   : "_print._sub._ipp._tcp"));
     if (error)
-      ERR("Error registering subtype for IPP printer %s (_print._sub._ipp._tcp or _universal._sub._ipp._tcp): %d",
-	  dnssd_name, error);
+      ERR("Error registering subtype for IPP printer %s (_print._sub._ipp._tcp "
+          "or _universal._sub._ipp._tcp): %d",
+          dnssd_name, error);
     else
-      NOTE("Registered subtype for IPP printer %s (_print._sub._ipp._tcp or _universal._sub._ipp._tcp).",
-	   dnssd_name);
+      NOTE(
+          "Registered subtype for IPP printer %s (_print._sub._ipp._tcp or "
+          "_universal._sub._ipp._tcp).",
+          dnssd_name);
   }
 
  /*
   * Finally _http.tcp (HTTP) for the web interface...
   */
 
-  error =
-    avahi_entry_group_add_service_strlst(g_options.dnssd_data->ipp_ref,
-					 (g_options.interface ?
-					  (int)if_nametoindex(g_options.interface) :
-					  AVAHI_IF_UNSPEC),
-					 AVAHI_PROTO_UNSPEC, 0,
-					 dnssd_name,
-					 "_http._tcp", NULL, NULL, g_options.real_port,
-					 NULL);
-  if (error)
+  error = avahi_entry_group_add_service_strlst(
+      g_options.dnssd_data->ipp_ref,
+      (g_options.interface ? (int)if_nametoindex(g_options.interface)
+                           : AVAHI_IF_UNSPEC),
+      AVAHI_PROTO_UNSPEC, 0, dnssd_name, "_http._tcp", NULL, NULL,
+      g_options.real_port, NULL);
+  if (error) {
     ERR("Error registering web interface of %s (_http._tcp): %d", dnssd_name,
 	error);
-  else {
+  } else {
     NOTE("Registered web interface of %s (_http._tcp).", dnssd_name);
-    error =
-      avahi_entry_group_add_service_subtype(g_options.dnssd_data->ipp_ref,
-					    (g_options.interface ?
-					     (int)if_nametoindex(g_options.interface) :
-					     AVAHI_IF_UNSPEC),
-					    AVAHI_PROTO_UNSPEC, 0,
-					    dnssd_name,
-					    "_http._tcp", NULL,
-					    "_printer._sub._http._tcp");
+    error = avahi_entry_group_add_service_subtype(
+        g_options.dnssd_data->ipp_ref,
+        (g_options.interface ? (int)if_nametoindex(g_options.interface)
+                             : AVAHI_IF_UNSPEC),
+        AVAHI_PROTO_UNSPEC, 0, dnssd_name, "_http._tcp", NULL,
+        "_printer._sub._http._tcp");
     if (error)
-      ERR("Error registering subtype for web interface of %s (_printer._sub._http._tcp): %d",
-	  dnssd_name, error);
+      ERR("Error registering subtype for web interface of %s "
+          "(_printer._sub._http._tcp): %d",
+          dnssd_name, error);
     else
-      NOTE("Registered subtype for web interface of %s (_printer._sub._http._tcp).",
-	   dnssd_name);
+      NOTE(
+          "Registered subtype for web interface of %s "
+          "(_printer._sub._http._tcp).",
+          dnssd_name);
   }
 
  /*
-  * Commit it...
+  * Commit it printer ...
   */
 
-  avahi_entry_group_commit(g_options.dnssd_data->ipp_ref);
+  // avahi_entry_group_commit(g_options.dnssd_data->ipp_ref);
 
-  avahi_string_list_free(ipp_txt);
+
+  pthread_create (&thread_escl, NULL, dnssd_escl_register, ipp_txt);
 
   return 0;
 }
 
-/*
- * 'dnssd_unregister()' - Unregister a printer object from DNS-SD.
- */
-
-void
-dnssd_unregister()
+void dnssd_unregister()
 {
   if (g_options.dnssd_data->ipp_ref) {
     avahi_entry_group_free(g_options.dnssd_data->ipp_ref);
     g_options.dnssd_data->ipp_ref = NULL;
   }
+  if (g_options.dnssd_data->uscan_ref) {
+    avahi_entry_group_free(g_options.dnssd_data->uscan_ref);
+    g_options.dnssd_data->uscan_ref = NULL;
+  }
 }
