@@ -15,6 +15,8 @@
 #pragma once
 #include <stdint.h>
 
+#include <pthread.h>
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -37,6 +39,8 @@ struct tcp_sock_t {
 struct tcp_conn_t {
   int sd;
   int is_closed;
+  int is_active;
+  pthread_mutex_t mutex;
 };
 
 struct tcp_sock_t *tcp_open(uint16_t, char* interface);
@@ -48,6 +52,10 @@ struct tcp_conn_t *tcp_conn_select(struct tcp_sock_t *sock,
 				   struct tcp_sock_t *sock6);
 void tcp_conn_close(struct tcp_conn_t *);
 
-struct http_packet_t *tcp_packet_get(struct tcp_conn_t *,
-                                     struct http_message_t *);
+struct http_packet_t *tcp_packet_get(struct tcp_conn_t *);
 int tcp_packet_send(struct tcp_conn_t *, struct http_packet_t *);
+
+int poll_tcp_socket(struct tcp_conn_t *tcp);
+
+int get_is_active(struct tcp_conn_t *tcp);
+void set_is_active(struct tcp_conn_t *tcp, int val);
